package com.programaker.api

import android.os.Build
import android.util.Log
import com.google.gson.Gson
import org.json.JSONObject
import java.io.DataOutputStream
import java.io.FileNotFoundException
import java.io.InputStream
import java.io.InputStreamReader
import java.lang.reflect.Type
import java.net.HttpURLConnection
import java.net.URL
import java.net.URLConnection


class ProgramakerApi(private val ApiRoot: String="https://programaker.com/api") {
    private val LogTag: String = "ProgramakerApi"
    var token: String? = null

    // API
    fun check(): Boolean {
        val conn = URL(getCheckUrl()).openConnection() as HttpURLConnection
        if (conn == null){
            Log.e(LogTag, "URL Connection not established, set to NULL")
            return false
        }
        try {
            addAuthHeader(conn)
        } catch(ex: TokenNotFoundException) {
            return false
        }

        val result: ProgramakerCheckResult
        try {
            result = parseJson(conn.inputStream, ProgramakerCheckResult::class.java)
        } catch(ex: JsonParseException) {
            ex.logError(LogTag)
            return false
        } catch (ex: FileNotFoundException) {
            ex.logError(LogTag)
            return false
        }
        return result.success
    }

    fun login(user: String, password: String): String {
        val conn = URL(getLoginUrl()).openConnection() as HttpURLConnection
        conn.setRequestProperty("Content-Type", "application/json")

        conn.requestMethod = "POST";
        conn.doOutput = true;

        val postData = JSONObject()
        postData.put("username", user)
        postData.put("password", password)

        val wr = DataOutputStream(conn.outputStream)
        wr.writeBytes(postData.toString());
        wr.flush();
        wr.close();

        val result: ProgramakerLoginResult
        result = parseJson(conn.inputStream, ProgramakerLoginResult::class.java)
        return result.token
    }

    // Initialization
    init {
        // Disable connection reuse if necessary
        // HTTP connection reuse which was buggy pre-froyo
        if (Build.VERSION.SDK.toInt() < Build.VERSION_CODES.FROYO) {
            System.setProperty("http.keepAlive", "false")
        }
    }

    // Private functions
    private fun getCheckUrl(): String {
        return "$ApiRoot/v0/sessions/check"
    }

    private fun getLoginUrl(): String {
        return "$ApiRoot/v0/sessions/login"
    }

    private fun addAuthHeader(conn: URLConnection) {
        if (token != null) {
            conn.setRequestProperty("Authorization", token)
        }
        else {
            throw TokenNotFoundException()
        }
    }

    private fun <T>parseJson(content: InputStream, resultClass: Type): T {
        val reader = InputStreamReader(content)
        val gson = Gson()
        return gson.fromJson(reader, resultClass)
    }
}

private fun FileNotFoundException.logError(tag: String) {
    Log.e(tag, "Cannot open: ${this.message}")
}

class JsonParseException(private val content: String, private val cls: Type) : Exception() {
    fun logError(tag: String) {
        Log.e(tag, "Cannot JSON parse: ${this.content} as ${this.cls}")
    }
}
